@@ -50,6 +50,7 @@ QString QExtFileInfo::canonicalPath(const QString& path)
   {
     KURL u = KURL::fromPathOrURL(path).upURL();
     QString s = u.path(-1) + "/";
+    if (s == "//") s = "/";
     QString s2 = path.mid(s.length());
     s2 = QExtFileInfo::canonicalPath(s) + s2;
     return s2;
@@ -62,7 +63,7 @@ QString QExtFileInfo::homeDirPath()
 }
 
 /** create a relative short url based in baseURL*/
-KURL QExtFileInfo::toRelative(const KURL& _urlToConvert,const KURL& _baseURL)
+KURL QExtFileInfo::toRelative(const KURL& _urlToConvert,const KURL& _baseURL, bool resolveSymlinks)
 {
   KURL urlToConvert = _urlToConvert;
   KURL baseURL = _baseURL;
@@ -71,10 +72,17 @@ KURL QExtFileInfo::toRelative(const KURL& _urlToConvert,const KURL& _baseURL)
   {
     if (urlToConvert.isLocalFile())
     {
-      QString path = QExtFileInfo::canonicalPath(urlToConvert.path());
+      QString path;
+      if (resolveSymlinks)
+        path = QExtFileInfo::canonicalPath(urlToConvert.path());
+      else
+        path = urlToConvert.path();
       if (!path.isEmpty())
         urlToConvert.setPath(path);
-      path = QExtFileInfo::canonicalPath(baseURL.path());
+      if (resolveSymlinks)
+        path = QExtFileInfo::canonicalPath(baseURL.path());
+      else
+        path = baseURL.path();
       if (!path.isEmpty())
         baseURL.setPath(path);
     }
@@ -159,7 +167,7 @@ KURL::List QExtFileInfo::allFiles( const KURL& path, const QString& mask, QWidge
   return internalFileInfo.allFilesInternal(path, mask, window);
 }
 
-KURL::List QExtFileInfo::allFilesRelative( const KURL& path, const QString& mask, QWidget *window)
+KURL::List QExtFileInfo::allFilesRelative( const KURL& path, const QString& mask, QWidget *window, bool resolveSymlinks)
 {
   QExtFileInfo internalFileInfo;
   KURL::List r = internalFileInfo.allFilesInternal(path, mask, window);
@@ -167,7 +175,7 @@ KURL::List QExtFileInfo::allFilesRelative( const KURL& path, const QString& mask
   KURL::List::Iterator it;
   for ( it = r.begin(); it != r.end(); ++it )
   {
-    *it = QExtFileInfo::toRelative( *it, path );
+    *it = QExtFileInfo::toRelative( *it, path, resolveSymlinks );
   }
 
   return r;
