@@ -197,8 +197,6 @@ Document::Document(KTextEditor::Document *doc,
   delayedTextChangedEnabled = true;
   docUndoRedo = new undoRedo(this);
 
-  //each document remember wheter it has a entry in quantarc
-  m_backupEntry = false;
   //path of the backup copy file of the document
   m_backupPathValue = QString::null;
 
@@ -221,7 +219,7 @@ Document::Document(KTextEditor::Document *doc,
 
 //   connect(m_doc, SIGNAL(marksChanged()), this, SLOT(slotMarksChanged()));
   connect(m_doc, SIGNAL(markChanged(KTextEditor::Mark, KTextEditor::MarkInterfaceExtension::MarkChangeAction)), this, SLOT(slotMarkChanged(KTextEditor::Mark, KTextEditor::MarkInterfaceExtension::MarkChangeAction)));
-  
+
 }
 
 Document::~Document()
@@ -878,7 +876,7 @@ void Document::slotCharactersInserted(int line, int column, const QString& strin
       return;
     }
   }
-  
+
 
  if ( (string == ">") ||
       (string == "<") )
@@ -1052,17 +1050,46 @@ bool Document::xmlAutoCompletion(int line, int column, const QString & string)
          }
     else if ( string[0] == qConfig.attrValueQuotation )
           {
-          //we need to find the attribute name
-          QString textLine = editIf->textLine(line).left(column-1);
-          QString attribute = textLine.mid(textLine.findRev(' ')+1);
-          showCodeCompletions( getAttributeValueCompletions(tagName, attribute) );
-          handled = true;
+            //we need to find the attribute name
+            QString textLine = editIf->textLine(line).left(column-1);
+            QString attribute = textLine.mid(textLine.findRev(' ')+1);
+            if (attribute == "style" && completionDTD->insideDTDs.contains("css"))
+            {
+              completionDTD = DTDs::ref()->find("text/css");
+              completionRequested = true;
+              return scriptAutoCompletion(line, column + 1, string);
+            }
+            showCodeCompletions( getAttributeValueCompletions(tagName, attribute) );
+            handled = true;
           }
   } // else - we are inside of a tag
   if (!handled)
   {
+    //check if we are inside a style attribute, and use css autocompletion if we are
+    QString textLine = editIf->textLine(line);
+    textLine = textLine.left(column);
+    int pos = textLine.findRev('"');
+    if (pos != -1)
+    {
+      pos = textLine.findRev(' ', pos);
+      if (pos != -1)
+      {
+        textLine = textLine.mid(pos + 1);
+        pos = textLine.find('=');
+        if (pos != -1)
+        {
+          QString attribute = textLine.left(pos);
+          if (attribute == "style" && completionDTD->insideDTDs.contains("css"))
+          {
+            completionDTD = DTDs::ref()->find("text/css");
+            completionRequested = true;
+            return scriptAutoCompletion(line, column + 1, string);
+          }
+        }
+      }
+    }
     QString s = editIf->textLine(line).left(column + 1);
-    int pos = s.findRev('&');
+    pos = s.findRev('&');
     if (pos != -1)
     {
       //complete character codes
@@ -1070,7 +1097,7 @@ bool Document::xmlAutoCompletion(int line, int column, const QString & string)
       showCodeCompletions(getCharacterCompletions(s));
       handled = true;
     }
-  } 
+  }
  return handled;
 }
 
@@ -2066,10 +2093,18 @@ bool Document::xmlCodeCompletion(int line, int col)
       index = tag->valueIndexAtPos(line,col);
       if (index != -1)      //inside a value
       {
-        tag->attributeValuePos(index, bLine, bCol);
-        s = tag->attributeValue(index).left(col - bCol);
-        showCodeCompletions( getAttributeValueCompletions(tagName, tag->attribute(index), s) );
-        handled = true;
+        s = tag->attribute(index);
+        if (s == "style" && completionDTD->insideDTDs.contains("css"))
+        {
+          completionDTD = DTDs::ref()->find("text/css");
+          return scriptAutoCompletion(line, col, "");
+        } else
+        {
+          tag->attributeValuePos(index, bLine, bCol);
+          s = tag->attributeValue(index).left(col - bCol);
+          showCodeCompletions( getAttributeValueCompletions(tagName, tag->attribute(index), s) );
+          handled = true;
+        }
       } else
       {
         index = tag->attributeIndexAtPos(line,col);
@@ -2272,7 +2307,7 @@ void Document::slotTextChanged()
     kdDebug(24000) << "Delayed text changed called." << endl;
     //delay the handling, otherwise we may get wrong values for (line,column)
     QTimer::singleShot(0, this, SLOT(slotDelayedTextChanged()));
-    delayedTextChangedEnabled = false;  
+    delayedTextChangedEnabled = false;
   }
 }
 
@@ -2284,10 +2319,10 @@ void Document::slotDelayedTextChanged(bool forced)
      parser->setParsingNeeded(true);
      QTimer::singleShot(1000, this, SLOT(slotDelayedTextChanged()));
      reparseEnabled = false;
-     delayedTextChangedEnabled = false;	
+     delayedTextChangedEnabled = false;
      return;
    }
-	
+
     uint line, column;
     QString oldNodeName = "";
     Node *node;
@@ -2298,7 +2333,9 @@ void Document::slotDelayedTextChanged(bool forced)
       viewCursorIf->cursorPositionReal(&line, &column);
       node = parser->nodeAt(line, column, false);
       if (node &&
-          (node->tag->type==Tag::XmlTag || node->tag->type == Tag::XmlTagEnd) )
+          ((node->tag->type == Tag::XmlTag && !node->tag->single) ||
+            node->tag->type == Tag::XmlTagEnd)
+           )
       {
         Tag *tag;
         tag = new Tag(*node->tag);
@@ -2324,7 +2361,7 @@ void Document::slotDelayedTextChanged(bool forced)
       node = parser->nodeAt(line, column, false);
       if (node &&
           node->tag->nameSpace + node->tag->name != currentNode->tag->nameSpace + currentNode->tag->name &&
-          (node->tag->type == Tag::XmlTag || node->tag->type == Tag::XmlTagEnd) && node->tag->validXMLTag)
+          ((node->tag->type == Tag::XmlTag && !node->tag->single) || node->tag->type == Tag::XmlTagEnd) && node->tag->validXMLTag)
       {
         int bl, bc, bl2, bc2;
         node->tag->beginPos(bl, bc);
@@ -2341,7 +2378,7 @@ void Document::slotDelayedTextChanged(bool forced)
           previousNode = 0L;
         }
         if (bl == bl2 && bc == bc2 &&
-           (currentNode->tag->type == Tag::XmlTag || currentNode->tag->type == Tag::XmlTagEnd))
+           ((node->tag->type == Tag::XmlTag && !node->tag->single) || currentNode->tag->type == Tag::XmlTagEnd))
         {
           QString newName = node->tag->name;
           bool updateClosing = (currentNode->tag->type == Tag::XmlTag) && !newName.startsWith("!");
@@ -2354,7 +2391,7 @@ void Document::slotDelayedTextChanged(bool forced)
             node = node->previousSibling();
           while (node)
           {
-            if (node->tag->validXMLTag && (node->tag->type == Tag::XmlTag || node->tag->type == Tag::XmlTagEnd))
+            if (node->tag->validXMLTag && ((node->tag->type == Tag::XmlTag && !node->tag->single) || node->tag->type == Tag::XmlTagEnd))
             {
               if (node->tag->nameSpace + node->tag->name == currentNode->tag->nameSpace + currentNode->tag->name )
               {
@@ -2415,7 +2452,7 @@ void Document::slotDelayedTextChanged(bool forced)
       StructTreeView::ref()->slotReparse(this, baseNode , qConfig.expandLevel);
     }
     reparseEnabled = true;
-   delayedTextChangedEnabled = true;	
+   delayedTextChangedEnabled = true;
 }
 
 /** Returns list of values for attribute */
@@ -2558,11 +2595,6 @@ void Document::clearErrorMarks()
   }
 }
 
-/** obvious */
-void Document::setBackupEntry(bool b)
-{
-   m_backupEntry = b;
-}
 QString Document::backupPathEntryValue()
 {
   return m_backupPathValue;
@@ -2576,59 +2608,68 @@ void Document::setBackupPathEntryValue(const QString& ev)
 /** if the document is modified then backup it and insert an entry in quantarc */
 void Document::createBackup(KConfig* config)
 {
-    if(isModified())
+  if (isModified())
+  {
+    if (isUntitled())
+    {
+      m_backupPathValue = qConfig.backupDirPath + untitledUrl + "." + hashFilePath("file:///" + untitledUrl) + "U";
+    } else
+    {
+      m_backupPathValue = qConfig.backupDirPath + url().fileName() + "." + hashFilePath(url().url());
+    }
+    QString backupPathValueURL = KURL::fromPathOrURL(m_backupPathValue).url();
+
+    //the encoding used for the current document
+    QString encoding = quantaApp->defaultEncoding();
+    if (encodingIf)
+      encoding = encodingIf->encoding();
+    if (encoding.isEmpty())
+      encoding = "utf8";  //final fallback
+
+    //creates an entry string in quantarc if it does not exist yet
+    config->setGroup("General Options");
+    QStringList backedupFilesEntryList = QuantaCommon::readPathListEntry(config, "List of backedup files"); //the files that were backedup
+    QStringList autosavedFilesEntryList = QuantaCommon::readPathListEntry(config, "List of autosaved files"); //the list of actual backup files inside $KDEHOME/share/apps/quanta/backups
+    if (!autosavedFilesEntryList.contains(backupPathValueURL)) //not yet backed up, add an entry for this file
+    {
+      autosavedFilesEntryList.append(backupPathValueURL);
+      config->writePathEntry("List of autosaved files", autosavedFilesEntryList);
+      if (!isUntitled())
+        backedupFilesEntryList.append(KURL::fromPathOrURL(url().path() + "." + qConfig.quantaPID).url());
+      else
+        backedupFilesEntryList.append(url().url() + "." + qConfig.quantaPID);
+      config->writePathEntry("List of backedup files", backedupFilesEntryList);
+      config->sync();
+    }
+
+    //creates a copy of this specific document
+    QFile file(m_backupPathValue);
+    if (file.open(IO_WriteOnly))
     {
-     if(isUntitled())
-      m_backupPathValue = qConfig.backupDirPath + url().fileName() + "." + hashFilePath(url().path())+"U";
-     else
-      m_backupPathValue = qConfig.backupDirPath + url().fileName() + "." + hashFilePath(url().path());
-     //creates an entry string in quantarc if it does not exist yet
-     config->setGroup("General Options");
-     QStringList backedupFilesEntryList = config->readPathListEntry("List of backedup files");
-     QStringList autosavedFilesEntryList = config->readPathListEntry("List of autosaved files");
-     QString encoding = quantaApp->defaultEncoding();
-     if (encodingIf)
-         encoding = encodingIf->encoding();
-     if (encoding.isEmpty())
-         encoding = "utf8";  //final fallback
-    if(!autosavedFilesEntryList.contains(m_backupPathValue))
-     {
-       autosavedFilesEntryList.append(m_backupPathValue);
-       config->writePathEntry("List of autosaved files", autosavedFilesEntryList);
-       backedupFilesEntryList.append(url().path() + "." + qConfig.quantaPID);
-       config->writePathEntry("List of backedup files", backedupFilesEntryList);
-       config->sync();
-       setBackupEntry(true);
-     }
-     //creates a copy of this specific document
-     QFile file(m_backupPathValue);
-     if(file.open(IO_WriteOnly))
-     {
       QTextStream stream(&file);
       stream.setCodec(QTextCodec::codecForName(encoding));
       stream << editIf->text();
       file.close();
-     }
-
     }
+  }
 }
-/** if there is no more need of a backup copy then remove it */
+/** if there is no more need for a backup copy then remove it */
 void Document::removeBackup(KConfig *config)
 {
+  QString backupPathValueURL = KURL::fromPathOrURL(m_backupPathValue).url();
+  
   config->reparseConfiguration();
   config->setGroup("General Options");
 
-  QStringList backedupFilesEntryList = config->readPathListEntry("List of backedup files");
-  QStringList autosavedFilesEntryList = config->readPathListEntry("List of autosaved files");
+  QStringList backedupFilesEntryList = QuantaCommon::readPathListEntry(config, "List of backedup files");
+  QStringList autosavedFilesEntryList = QuantaCommon::readPathListEntry(config, "List of autosaved files");
 
-  autosavedFilesEntryList.remove(m_backupPathValue);
-  config->writePathEntry("List of autosaved files",autosavedFilesEntryList);
-  backedupFilesEntryList.remove(url().path() + "." + qConfig.quantaPID);
+  autosavedFilesEntryList.remove(backupPathValueURL);
+  config->writePathEntry("List of autosaved files", autosavedFilesEntryList);
+  backedupFilesEntryList.remove(KURL::fromPathOrURL(url().path() + "." + qConfig.quantaPID).url());
   config->writePathEntry("List of backedup files", backedupFilesEntryList);
   config->sync();
 
-  setBackupEntry(false);
-
   if(QFile::exists(m_backupPathValue))
     QFile::remove(m_backupPathValue);
 }
@@ -2918,12 +2959,21 @@ void Document::processDTD(const QString& documentType)
       }
    } else //DOCTYPE not found in file
    {
-     QString mimetype = KMimeType::findByURL(url())->name();
-     const DTDStruct *currdtd = DTDs::ref()->DTDfromMimeType(mimetype);
-     if (currdtd)
-        setDTDIdentifier(currdtd->name);
-     else
-        setDTDIdentifier(projectDTD);
+     KURL u = url();
+     QString dtdId = DTDs::ref()->DTDforURL(u)->name;
+//     if (dtdId == "empty")
+     {
+       const DTDStruct * dtd = DTDs::ref()->find(projectDTD);
+       if (DTDs::canHandle(dtd, u))
+        dtdId = projectDTD;
+       else
+       {
+         dtd = DTDs::ref()->find(qConfig.defaultDocType);
+         if (DTDs::canHandle(dtd, u))
+           dtdId = qConfig.defaultDocType;
+       }
+     }
+     setDTDIdentifier(dtdId);
    }
  } else //dtdName is read from the method's parameter
  {
